package org.openjdk.gcbench.retain;

import org.openjdk.jmh.annotations.*;

import java.lang.ref.PhantomReference;
import java.lang.ref.Reference;
import java.lang.ref.ReferenceQueue;
import java.lang.ref.SoftReference;
import java.util.TreeMap;
import java.util.concurrent.TimeUnit;

@Warmup(iterations = 5, time = 1, timeUnit = TimeUnit.SECONDS)
@Measurement(iterations = 5, time = 1, timeUnit = TimeUnit.SECONDS)
@Fork(1)
@BenchmarkMode(Mode.AverageTime)
@OutputTimeUnit(TimeUnit.NANOSECONDS)
@Threads(Threads.MAX)
@State(Scope.Benchmark)
public class LargeGraphPhantomRef {

    PhantomReference<TreeMap<String, Object>> ref;
    ReferenceQueue<Object> rq;

    @Param({"100000"})
    private int size;

    public TreeMap<String, Object> createTreeMap() {
        TreeMap<String, Object> t = new TreeMap<>();
        for (int c = 0; c < size; c++) {
            t.put("String" + c, new Object());
        }
        return t;
    }

    @Setup
    public void setup() {
        rq = new ReferenceQueue<>();
        ref = new PhantomReference<>(createTreeMap(), rq);
    }

    @Benchmark
    public Object test() {
        if (rq.poll() != null) {
            ref = new PhantomReference<>(createTreeMap(), rq);
        }
        return new byte[100];
    }

}
