package org.openjdk.gcbench.retain;

import org.openjdk.jmh.annotations.*;

import java.lang.ref.Reference;
import java.lang.ref.ReferenceQueue;
import java.lang.ref.SoftReference;
import java.lang.ref.WeakReference;
import java.util.ArrayList;
import java.util.List;
import java.util.WeakHashMap;
import java.util.concurrent.TimeUnit;

@Warmup(iterations = 5, time = 1, timeUnit = TimeUnit.SECONDS)
@Measurement(iterations = 5, time = 1, timeUnit = TimeUnit.SECONDS)
@Fork(1)
@BenchmarkMode(Mode.AverageTime)
@OutputTimeUnit(TimeUnit.NANOSECONDS)
@Threads(Threads.MAX)
@State(Scope.Benchmark)
public class SoftRefs {

    Reference[] retain;
    ReferenceQueue<Object> rq;

    @Param({"100000"})
    private int size;

    @Setup
    public void setup() {
        rq = new ReferenceQueue<>();
        Reference[] list = new Reference[size];
        for (int c = 0; c < size; c++) {
            list[c] = new SoftReference<>(new Object(), rq);
        }
        retain = list;
    }

    @Benchmark
    public Object test() {
        while (rq.poll() != null); // drain
        // allocation pressure to trigger GCs
        return new Object();
    }

}
